@@ -20,11 +20,15 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef _HANDYBRL_H
-#define _HANDYBRL_H			
+#ifndef __HANDYBRL_H__
+#define __HANDYBRL_H__			
 
+#include <glib.h>
 #include "braille.h"
 
-int handy_brl_open_device (char* DeviceName, short Port, BRL_DEV_CALLBACK DeviceCallback, BRL_DEVICE *Device);
+gint handy_brl_open_device (gchar               *device_name,
+			   gshort               port,
+			   BRLDevCallback       device_callback,
+			   BRLDevice            *device);
 
 #endif
